package com.logicbig.example;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.ElementFilter;
import javax.tools.Diagnostic;
import java.util.List;
import java.util.Set;

@SupportedAnnotationTypes("*")
@SupportedSourceVersion(SourceVersion.RELEASE_8)
public class EqualMethodProcessor extends AbstractProcessor {
    @Override
    public boolean process(Set<? extends TypeElement> annotations,
                           RoundEnvironment roundEnv) {

        ElementFilter.typesIn(roundEnv.getRootElements())
                .stream()
                .filter(this::implementsDataInterface)
                .forEach(typeElement -> {

            List<ExecutableElement> methods =
                    ElementFilter.methodsIn(typeElement.getEnclosedElements());

            if (!methods.stream().anyMatch(m -> m.getSimpleName().contentEquals("equals"))) {
                processingEnv.getMessager().printMessage(Diagnostic.Kind.ERROR,
                        "'equals' method must be overridden for the class implementing " +
                                "com.logicbig.example.Data. \n Error class: " + typeElement);
            }
        });
        return false;
    }

    private boolean implementsDataInterface(TypeElement typeElement) {
        List<? extends TypeMirror> interfaces = typeElement.getInterfaces();
        return interfaces.stream().anyMatch(theMirror ->
                theMirror.toString().equals(Data.class.getName()));
    }
}